@@ -3,19 +3,23 @@ use std::collections::{HashMap, VecDeque};
 
 use anyhow::{Context as _, Result};
 use inflector::Inflector;
-use proc_macro2::{Literal, TokenStream};
+use proc_macro2::TokenStream;
 use quote::quote;
 
-use ethers_core::abi::{
-    param_type::Reader,
-    struct_def::{FieldDeclaration, FieldType, StructFieldType, StructType},
-    ParamType, SolStruct,
+use ethers_core::{
+    abi::{
+        param_type::Reader,
+        struct_def::{FieldDeclaration, FieldType, StructFieldType, StructType},
+        ParamType, SolStruct,
+    },
+    macros::ethers_contract_crate,
 };
 
-use crate::contract::{types, Context};
-use crate::rawabi::{Component, RawAbi};
-use crate::util;
-use std::any::Any;
+use crate::{
+    contract::{types, Context},
+    rawabi::{Component, RawAbi},
+    util,
+};
 
 impl Context {
     /// Generate corresponding types for structs parsed from a human readable ABI
@@ -31,6 +35,41 @@ impl Context {
         }
     }
 
+    /// In the event of type conflicts this allows for removing a specific struct type.
+    pub fn remove_struct(&mut self, name: &str) {
+        if self.human_readable {
+            self.abi_parser.structs.remove(name);
+        } else {
+            self.internal_structs.structs.remove(name);
+        }
+    }
+
+    /// Returns the type definition for the struct with the given name
+    pub fn struct_definition(&mut self, name: &str) -> Result<TokenStream> {
+        if self.human_readable {
+            self.generate_human_readable_struct(name)
+        } else {
+            self.generate_internal_struct(name)
+        }
+    }
+
+    /// Generates the type definition for the name that matches the given identifier
+    fn generate_internal_struct(&self, id: &str) -> Result<TokenStream> {
+        let sol_struct = self.internal_structs.structs.get(id).context("struct not found")?;
+        let struct_name = self
+            .internal_structs
+            .rust_type_names
+            .get(id)
+            .context(format!("No types found for {}", id))?;
+        let tuple = self
+            .internal_structs
+            .struct_tuples
+            .get(id)
+            .context(format!("No types found for {}", id))?
+            .clone();
+        self.expand_internal_struct(struct_name, sol_struct, tuple)
+    }
+
     /// Returns the `TokenStream` with all the internal structs extracted form the JSON ABI
     fn gen_internal_structs(&self) -> Result<TokenStream> {
         let mut structs = TokenStream::new();
@@ -38,19 +77,7 @@ impl Context {
         ids.sort();
 
         for id in ids {
-            let sol_struct = &self.internal_structs.structs[id];
-            let struct_name = self
-                .internal_structs
-                .rust_type_names
-                .get(id)
-                .context(format!("No types found for {}", id))?;
-            let tuple = self
-                .internal_structs
-                .struct_tuples
-                .get(id)
-                .context(format!("No types found for {}", id))?
-                .clone();
-            structs.extend(self.expand_internal_struct(struct_name, sol_struct, tuple)?);
+            structs.extend(self.generate_internal_struct(id)?);
         }
         Ok(structs)
     }
@@ -79,17 +106,13 @@ impl Context {
                         "Mapping types in struct `{}` are not supported {:?}",
                         name,
                         field
-                    ));
+                    ))
                 }
             }
         }
 
         let sig = if let ParamType::Tuple(ref tokens) = tuple {
-            tokens
-                .iter()
-                .map(|kind| kind.to_string())
-                .collect::<Vec<_>>()
-                .join(",")
+            tokens.iter().map(|kind| kind.to_string()).collect::<Vec<_>>().join(",")
         } else {
             "".to_string()
         };
@@ -100,11 +123,72 @@ impl Context {
 
         let name = util::ident(name);
 
+        // use the same derives as for events
+        let derives = util::expand_derives(&self.event_derives);
+
+        let ethers_contract = ethers_contract_crate();
+        Ok(quote! {
+            #abi_signature_doc
+            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #derives)]
+            pub struct #name {
+                #( #fields ),*
+            }
+        })
+    }
+
+    fn generate_human_readable_struct(&self, name: &str) -> Result<TokenStream> {
+        let sol_struct = self.abi_parser.structs.get(name).context("struct not found")?;
+        let mut fields = Vec::with_capacity(sol_struct.fields().len());
+        let mut param_types = Vec::with_capacity(sol_struct.fields().len());
+        for field in sol_struct.fields() {
+            let field_name = util::ident(&field.name().to_snake_case());
+            match field.r#type() {
+                FieldType::Elementary(ty) => {
+                    param_types.push(ty.clone());
+                    let ty = types::expand(ty)?;
+                    fields.push(quote! { pub #field_name: #ty });
+                }
+                FieldType::Struct(struct_ty) => {
+                    let ty = expand_struct_type(struct_ty);
+                    fields.push(quote! { pub #field_name: #ty });
+
+                    let name = struct_ty.name();
+                    let tuple = self
+                        .abi_parser
+                        .struct_tuples
+                        .get(name)
+                        .context(format!("No types found for {}", name))?
+                        .clone();
+                    let tuple = ParamType::Tuple(tuple);
+
+                    param_types.push(struct_ty.as_param(tuple));
+                }
+                FieldType::Mapping(_) => {
+                    return Err(anyhow::anyhow!(
+                        "Mapping types in struct `{}` are not supported {:?}",
+                        name,
+                        field
+                    ))
+                }
+            }
+        }
+
+        let abi_signature = format!(
+            "{}({})",
+            name,
+            param_types.iter().map(|kind| kind.to_string()).collect::<Vec<_>>().join(","),
+        );
+
+        let abi_signature_doc = util::expand_doc(&format!("`{}`", abi_signature));
+
+        let name = util::ident(name);
+
         // use the same derives as for events
         let derives = &self.event_derives;
         let derives = quote! {#(#derives),*};
 
-        let ethers_contract = util::ethers_contract_crate();
+        let ethers_contract = ethers_contract_crate();
+
         Ok(quote! {
             #abi_signature_doc
             #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #derives)]
@@ -120,69 +204,7 @@ impl Context {
         let mut names: Vec<_> = self.abi_parser.structs.keys().collect();
         names.sort();
         for name in names {
-            let sol_struct = &self.abi_parser.structs[name];
-            let mut fields = Vec::with_capacity(sol_struct.fields().len());
-            let mut param_types = Vec::with_capacity(sol_struct.fields().len());
-            for field in sol_struct.fields() {
-                let field_name = util::ident(&field.name().to_snake_case());
-                match field.r#type() {
-                    FieldType::Elementary(ty) => {
-                        param_types.push(ty.clone());
-                        let ty = types::expand(ty)?;
-                        fields.push(quote! { pub #field_name: #ty });
-                    }
-                    FieldType::Struct(struct_ty) => {
-                        let ty = expand_struct_type(struct_ty);
-                        fields.push(quote! { pub #field_name: #ty });
-
-                        let name = struct_ty.name();
-                        let tuple = self
-                            .abi_parser
-                            .struct_tuples
-                            .get(name)
-                            .context(format!("No types found for {}", name))?
-                            .clone();
-                        let tuple = ParamType::Tuple(tuple);
-
-                        param_types.push(struct_ty.as_param(tuple));
-                    }
-                    FieldType::Mapping(_) => {
-                        return Err(anyhow::anyhow!(
-                            "Mapping types in struct `{}` are not supported {:?}",
-                            name,
-                            field
-                        ));
-                    }
-                }
-            }
-
-            let abi_signature = format!(
-                "{}({})",
-                name,
-                param_types
-                    .iter()
-                    .map(|kind| kind.to_string())
-                    .collect::<Vec<_>>()
-                    .join(","),
-            );
-
-            let abi_signature_doc = util::expand_doc(&format!("`{}`", abi_signature));
-
-            let name = util::ident(name);
-
-            // use the same derives as for events
-            let derives = &self.event_derives;
-            let derives = quote! {#(#derives),*};
-
-            let ethers_contract = util::ethers_contract_crate();
-
-            structs.extend(quote! {
-            #abi_signature_doc
-            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #derives)]
-            pub struct #name {
-                #( #fields ),*
-            }
-        });
+            structs.extend(self.generate_human_readable_struct(name)?);
         }
         Ok(structs)
     }
@@ -190,29 +212,29 @@ impl Context {
 
 /// Helper to match `ethabi::Param`s with structs and nested structs
 ///
-/// This is currently used to get access to all the unique solidity structs used as function in/output until `ethabi` supports it as well.
+/// This is currently used to get access to all the unique solidity structs used as function
+/// in/output until `ethabi` supports it as well.
 #[derive(Debug, Clone, Default)]
 pub struct InternalStructs {
-    /// All unique internal types that are function inputs or outputs
-    top_level_internal_types: HashMap<String, Component>,
-
-    /// (function name, param name) -> struct which are the identifying properties we get the name from ethabi.
-    function_params: HashMap<(String, String), String>,
+    /// (function name, param name) -> struct which are the identifying properties we get the name
+    /// from ethabi.
+    pub(crate) function_params: HashMap<(String, String), String>,
 
     /// (function name) -> Vec<structs> all structs the function returns
-    outputs: HashMap<String, Vec<String>>,
+    pub(crate) outputs: HashMap<String, Vec<String>>,
 
     /// All the structs extracted from the abi with their identifier as key
-    structs: HashMap<String, SolStruct>,
+    pub(crate) structs: HashMap<String, SolStruct>,
 
     /// solidity structs as tuples
-    struct_tuples: HashMap<String, ParamType>,
+    pub(crate) struct_tuples: HashMap<String, ParamType>,
 
     /// Contains the names for the rust types (id -> rust type name)
-    rust_type_names: HashMap<String, String>,
+    pub(crate) rust_type_names: HashMap<String, String>,
 }
 
 impl InternalStructs {
+    /// Creates a new instance with a filled type mapping table based on the abi
     pub fn new(abi: RawAbi) -> Self {
         let mut top_level_internal_types = HashMap::new();
         let mut function_params = HashMap::new();
@@ -269,7 +291,6 @@ impl InternalStructs {
         }
 
         Self {
-            top_level_internal_types,
             function_params,
             outputs,
             structs,
@@ -290,9 +311,16 @@ impl InternalStructs {
             .and_then(|id| self.rust_type_names.get(id))
             .map(String::as_str)
     }
+
+    /// Returns the mapping table of abi `internal type identifier -> rust type`
+    pub fn rust_type_names(&self) -> &HashMap<String, String> {
+        &self.rust_type_names
+    }
 }
 
-/// This will determine the name of the rust type and will make sure that possible collisions are resolved by adjusting the actual Rust name of the structure, e.g. `LibraryA.Point` and `LibraryB.Point` to `LibraryAPoint` and `LibraryBPoint`.
+/// This will determine the name of the rust type and will make sure that possible collisions are
+/// resolved by adjusting the actual Rust name of the structure, e.g. `LibraryA.Point` and
+/// `LibraryB.Point` to `LibraryAPoint` and `LibraryBPoint`.
 fn insert_rust_type_name(
     type_names: &mut HashMap<String, (String, Vec<String>)>,
     mut name: String,
@@ -303,11 +331,7 @@ fn insert_rust_type_name(
         let mut other_name = name.clone();
         // name collision `A.name` `B.name`, rename to `AName`, `BName`
         if !other_projections.is_empty() {
-            other_name = format!(
-                "{}{}",
-                other_projections.remove(0).to_pascal_case(),
-                other_name
-            );
+            other_name = format!("{}{}", other_projections.remove(0).to_pascal_case(), other_name);
         }
         insert_rust_type_name(type_names, other_name, other_projections, other_id);
 
@@ -322,7 +346,10 @@ fn insert_rust_type_name(
 
 /// Tries to determine the `ParamType::Tuple` for every struct.
 ///
-/// If a structure has nested structures, these must be determined first, essentially starting with structures consisting of only elementary types before moving on to higher level structures, for example `Proof {point: Point}, Point {x:int, y:int}` start by converting Point into a tuple of `x` and `y` and then substituting `point` with this within `Proof`.
+/// If a structure has nested structures, these must be determined first, essentially starting with
+/// structures consisting of only elementary types before moving on to higher level structures, for
+/// example `Proof {point: Point}, Point {x:int, y:int}` start by converting Point into a tuple of
+/// `x` and `y` and then substituting `point` with this within `Proof`.
 fn resolve_struct_tuples(all_structs: &HashMap<String, SolStruct>) -> HashMap<String, ParamType> {
     let mut params = HashMap::new();
     let mut structs: VecDeque<_> = all_structs.iter().collect();
@@ -331,7 +358,7 @@ fn resolve_struct_tuples(all_structs: &HashMap<String, SolStruct>) -> HashMap<St
     let mut sequential_retries = 0;
     'outer: while let Some((id, ty)) = structs.pop_front() {
         if sequential_retries > structs.len() {
-            break;
+            break
         }
         if let Some(tuple) = ty.as_tuple() {
             params.insert(id.to_string(), tuple);
@@ -361,7 +388,7 @@ fn resolve_struct_tuples(all_structs: &HashMap<String, SolStruct>) -> HashMap<St
                             // struct field needs to be resolved first
                             structs.push_back((id, ty));
                             sequential_retries += 1;
-                            continue 'outer;
+                            continue 'outer
                         }
                     }
                     _ => {
@@ -378,27 +405,21 @@ fn resolve_struct_tuples(all_structs: &HashMap<String, SolStruct>) -> HashMap<St
     params
 }
 
-/// turns the tuple component into a struct if it's still missing in the map, including all inner structs
+/// turns the tuple component into a struct if it's still missing in the map, including all inner
+/// structs
 fn insert_structs(structs: &mut HashMap<String, SolStruct>, tuple: &Component) {
     if let Some(internal_ty) = tuple.internal_type.as_ref() {
         let ident = struct_type_identifier(internal_ty);
         if structs.contains_key(ident) {
-            return;
+            return
         }
         if let Some(fields) = tuple
             .components
             .iter()
-            .map(|f| {
-                Reader::read(&f.type_field)
-                    .ok()
-                    .and_then(|kind| field(structs, f, kind))
-            })
+            .map(|f| Reader::read(&f.type_field).ok().and_then(|kind| field(structs, f, kind)))
             .collect::<Option<Vec<_>>>()
         {
-            let s = SolStruct {
-                name: ident.to_string(),
-                fields,
-            };
+            let s = SolStruct { name: ident.to_string(), fields };
             structs.insert(ident.to_string(), s);
         }
     }
@@ -477,10 +498,7 @@ fn struct_type_name(name: &str) -> &str {
 
 /// `Pairing.G2Point` -> `Pairing.G2Point`
 fn struct_type_identifier(name: &str) -> &str {
-    name.trim_start_matches("struct ")
-        .split('[')
-        .next()
-        .unwrap()
+    name.trim_start_matches("struct ").split('[').next().unwrap()
 }
 
 /// `struct Pairing.Nested.G2Point[]` -> `[Pairing, Nested]`
