@@ -1,12 +1,10 @@
 use anyhow::{anyhow, Result};
-use ethers_core::abi::ParamType;
+use ethers_core::{abi::ParamType, macros::ethers_core_crate};
 use proc_macro2::{Literal, TokenStream};
 use quote::quote;
 
-use super::util;
-
 pub(crate) fn expand(kind: &ParamType) -> Result<TokenStream> {
-    let ethers_core = util::ethers_core_crate();
+    let ethers_core = ethers_core_crate();
 
     match kind {
         ParamType::Address => Ok(quote! { #ethers_core::types::Address }),
@@ -49,13 +47,10 @@ pub(crate) fn expand(kind: &ParamType) -> Result<TokenStream> {
         }
         ParamType::Tuple(members) => {
             if members.is_empty() {
-                return Err(anyhow!("Tuple must have at least 1 member"));
+                return Err(anyhow!("Tuple must have at least 1 member"))
             }
 
-            let members = members
-                .iter()
-                .map(|member| expand(member))
-                .collect::<Result<Vec<_>, _>>()?;
+            let members = members.iter().map(expand).collect::<Result<Vec<_>, _>>()?;
             Ok(quote! { (#(#members,)*) })
         }
     }
